from typing import *
from flatten_list.flatten_list import flatten
from semantic_version import Version

VERSION = Version("0.10.0")

RELEASES_RSS_URL = "https://pypi.org/rss/project/ideaseed/releases.xml"

COLOR_NAME_TO_HEX_MAP: Dict[str, int] = {
    "Blue": 0xAECBFA,
    "Brown": 0xE6C9A8,
    "DarkBlue": 0xAECBFA,
    "Gray": 0xE8EAED,
    "Green": 0xCCFF90,
    "Orange": 0xFBBC04,
    "Pink": 0xFDCFE8,
    "Purple": 0xD7AEFB,
    "Red": 0xF28B82,
    "Teal": 0xA7FFEB,
    "White": 0xFFFFFF,
    "Yellow": 0xFFF475,
}

COLOR_ALIASES = {
    "Cyan": "Teal",
    "Indigo": "DarkBlue",
    "Grey": "Gray",
    "Magenta": "Purple",
}

VALID_COLOR_NAMES = list(flatten(list(COLOR_ALIASES.items()))) + list(
    COLOR_NAME_TO_HEX_MAP.keys()
)

# colors
C_PRIMARY = 0x268CCE



from ideaseed.constants import COLOR_NAME_TO_HEX_MAP, C_PRIMARY
from typing import *
from github.Repository import Repository
from github.Label import Label
from ideaseed.utils import dye, english_join, readable_text_color_on
from wcwidth import wcswidth
import textwrap
import cli_box
from strip_ansi import strip_ansi

ABOUT_SCREEN = """


                ██████████  ██  ██
                ██      ██  ██  ██
                ██████████  ██  ██
                ██      ██  ██  ██
                ██████████████████
                ██      ██      ██
                ██████████      ██
                ██      ██      ██
                ██████████      ██
      
            ideaseed v{version}
            by Ewen Le Bihan
            more at https://ewen.works
      
                ~ thx to these ppl ~

https://github.com/kiwiz
    This madman reverse-engineered Google Keep's
    internal REST API so that anyone could 
    use it with ease.
                    
https://github.com/PyGithub
    This one of the cleanest libs I've ever used.
    Like `requests`-level cleanliness.
                    
https://github.com/docopt
    Designing CLIs with this is a fucking breeze.
    I can almost copy-paste documentation into
    a docstring and call it a day, it's crazy.
"""

CARD_INNER_WIDTH = 50
CARD_LINE_SEPARATOR = "─" * CARD_INNER_WIDTH

ISSUE_ART = """\
Opening issue in {owner} › {repository} › {project} › {column}...

{issue_card}{timeline_item_milestone}{timeline_item_assignees}
 │
[◫] @{username} added this to {column} in {project}
 │
 →  Issue available at {url}
"""

GITHUB_ISSUE_TIMELINE_ITEM_MILESTONE_ART = """
 │
[⚐] @{username} added this to the {title} milestone"""

GITHUB_ISSUE_TIMELINE_ITEM_ASSIGNEES_ART = """
 │
[Ω] @{username} {assignation_sentence}"""

GITHUB_CARD_ART = """\
{card_header}
{content}
"""

GITHUB_ART = """\
{card}
 │
 →  Project card available at {url}
"""


def strwidth(o: str) -> int:
    """
    Smartly calculates the actual width taken on a terminal of `o`. 
    Handles ANSI codes (using `strip-ansi`) and Unicode (using `wcwidth`)
    """
    return wcswidth(strip_ansi(o))


def make_card_header(left: str, right: str) -> str:
    """
    Returns
    "{left}{enough spaces}{right}"
    
    Truncates `left` to leave enough spaces.
    spacing is at a minimum of two columns.
    """
    left = str(left)
    right = str(right)
    # Card width
    # Calculate max length of title
    left_max_len = CARD_INNER_WIDTH - strwidth(right) - 2
    if strwidth(left) >= left_max_len:
        left = f"{left:.{left_max_len-1}}" + "…"

    spaces = " " * (CARD_INNER_WIDTH - strwidth(left) + 2 - strwidth(right))

    return left + spaces + right


ISSUE_CARD_ART = """\
{card_header}

{content}{labels}
"""

TAGS_ART = "[{tag}]"

GOOGLE_KEEP_ART = """\
Adding card to your Google Keep account:

{card}
 │
 →  Available at {url}
"""

GOOGLE_KEEP_CARD_ART = """\
{card_header}

{content}{tags}
"""

GOOGLE_KEEP_PINNED = "⚲"


def make_google_keep_art(
    title: Optional[str],
    pinned: bool,
    tags: List[str],
    url: str,
    body: str,
    color: str,
) -> str:
    hex_color = COLOR_NAME_TO_HEX_MAP.get(color)
    card_header = make_card_header(
        left=dye(title or "", style="bold"), right=GOOGLE_KEEP_PINNED if pinned else ""
    )
    card = cli_box.rounded(
        GOOGLE_KEEP_CARD_ART.format(
            card_header=card_header,
            content=wrap_card_content(body),
            tags="\n\n" + "  ".join([format_label(t, hex_color or 0xDDD) for t in tags])
            if tags
            else "",
        ),
        align="left",
    )

    card = "\n".join([dye(line, fg=hex_color) for line in card.split("\n")])
    return GOOGLE_KEEP_ART.format(card=card, url=url)


def wrap_card_content(body: str) -> str:
    return textwrap.fill(body, width=CARD_INNER_WIDTH, replace_whitespace=False)


def make_github_project_art(
    owner: str, repository: str, project: str, column: str, body: str, url: str
):
    card_header = make_card_header(
        left=f"{owner}/{repository}", right=f"{column} in {project}"
    )
    card = cli_box.rounded(
        GITHUB_CARD_ART.format(
            content=wrap_card_content(body), card_header=dye(card_header, style="dim")
        ),
        align="left",
    )
    return GITHUB_ART.format(card=card, url=url)


def make_github_user_project_art(
    username: str, project: str, column: str, body: str, url: str
):
    card_header = make_card_header(left=f"@{username}", right=f"{column} in {project}")
    card = cli_box.rounded(
        GITHUB_CARD_ART.format(
            content=wrap_card_content(body), card_header=dye(card_header, style="dim")
        ),
        align="left",
    )
    return GITHUB_ART.format(card=card, url=url)


def make_github_issue_art(
    owner: str,
    repository: str,
    project: str,
    column: str,
    username: str,
    url: str,
    issue_number: Union[str, int],
    labels: List[Label],
    body: str,
    title: Optional[str],
    assignees: List[str],
    milestone: Optional[str],
) -> str:
    card_header = make_card_header(
        left=dye(title or "", style="bold"), right=dye(f"#{issue_number}", fg=C_PRIMARY)
    )
    timeline_item_milestone = ""
    if milestone:
        timeline_item_milestone = GITHUB_ISSUE_TIMELINE_ITEM_MILESTONE_ART.format(
            username=username, title=dye(milestone, style="bold")
        )
    timeline_item_assignees = ""
    if assignees:
        timeline_item_assignees = GITHUB_ISSUE_TIMELINE_ITEM_ASSIGNEES_ART.format(
            username=username,
            assignation_sentence=(
                "self-assigned this"
                if assignees == [username]
                else (
                    "assigned "
                    + english_join([dye("@" + a, style="bold") for a in assignees])
                    + " to this"
                )
            ),
        )
    card = cli_box.rounded(
        ISSUE_CARD_ART.format(
            card_header=card_header,
            content=wrap_card_content(body),
            labels="\n\n"
            + "  ".join([format_label(l.name, int(l.color, 16)) for l in labels])
            if labels
            else "",
        ),
        align="left",
    )

    return ISSUE_ART.format(
        owner=dye(owner, fg=C_PRIMARY, style="bold"),
        repository=dye(repository, fg=C_PRIMARY, style="bold"),
        project=dye(project, fg=C_PRIMARY, style="bold"),
        column=dye(column, fg=C_PRIMARY, style="bold"),
        username=username,
        url=dye(url, style="bold"),
        issue_card=card,
        timeline_item_milestone=timeline_item_milestone,
        timeline_item_assignees=timeline_item_assignees,
    )


def format_label(
    name: str, color: int = 0xDDDDDD, text_color: Optional[int] = None
) -> str:
    """
    Renders a label using a color
    
    ``text_color`` defaults to ``readable_text_color_on(color)``
    
    WARN: ``color`` and ``text_color`` must be a 6-digit-long hex int representing the color.
    """
    text_color = readable_text_color_on(color) if text_color is None else text_color
    return dye(f" {name} ", bg=color, fg=text_color)



from github.Label import Label
from ideaseed.dumb_utf8_art import (
    make_github_issue_art,
    make_github_project_art,
    make_github_user_project_art,
)
import os
import re
import webbrowser
from os.path import dirname
from github.GithubException import BadCredentialsException, TwoFactorException
from ideaseed.utils import (
    ask,
    dye,
    get_token_cache_filepath,
    print_dry_run,
    error_message_no_object_found,
)
from random import randint
from ideaseed.constants import C_PRIMARY
from typing import *
from github import Github
import json
from github.GithubException import TwoFactorException
import inquirer as q
from ideaseed.utils import ask


def validate_label_color(answers: dict, color: str):
    """
    Throws a `inquirer.errors.ValidationError` when the format isn't matched.
    (format: 6-digit hex int representing a color)
    """
    if not re.match(r"[a-fA-F0-9]{6}", color):
        raise q.errors.ValidationError(
            "", reason="Please use a valid color (6-digit hexadecimal integer)"
        )


def login_with_cache() -> Optional[Github]:
    """
    Tries to login using the cached credentials.
    `None` is returned if the cache does not exist or is invalid.
    """
    if os.path.exists(get_token_cache_filepath("github")):
        with open(get_token_cache_filepath("github"), encoding="utf8") as file:
            creds = json.load(file)
            creds = (
                [creds["username"], creds["password"]]
                if creds["pat"] is None
                else [creds["pat"]]
            )
            try:
                gh = Github(*creds)
                gh.get_user().name
                return gh
            except BadCredentialsException:
                return None
            except TwoFactorException:
                return None
    return None


def clear_auth_cache():
    if os.path.exists(get_token_cache_filepath("github")):
        os.remove(get_token_cache_filepath("github"))


def write_auth_cache(auth):
    os.makedirs(dirname(get_token_cache_filepath("github")), exist_ok=True)
    with open(get_token_cache_filepath("github"), "w", encoding="utf8") as file:
        json.dump(auth, file)


def login(args: Dict[str, Any], method: Optional[str] = None) -> Github:
    """
    Returns a `Github` instance to interact with.
    Prompts the user to login, either via username/password
    or using a Personal Access Token
    """
    gh = login_with_cache()
    if gh is not None:
        return gh
    else:
        del gh

    questions = [
        q.List(
            name="method",
            message="Log in using",
            choices=["Personal Access Token", "Username and password"],
            ignore=lambda _: method is not None,
        ),
        q.Password(
            name="pat",
            message="Personal Access Token",
            ignore=lambda ans: (method or ans["method"]) != "Personal Access Token",
        ),
        q.Text(
            name="username",
            message="Username",
            ignore=lambda ans: (method or ans["method"]) != "Username and password",
        ),
        q.Password(
            name="password",
            message="Password",
            ignore=lambda ans: (method or ans["method"]) != "Username and password",
        ),
    ]
    answers = q.prompt(questions)

    if not args["--no-auth-cache"]:
        write_auth_cache(answers)

    if answers["pat"] is not None:
        try:
            gh = Github(answers["pat"])
            github_username(gh)
            return gh
        except BadCredentialsException:
            print("Bad token")
            return login(args, method)
    else:
        try:
            gh = Github(answers["username"], answers["password"])
            github_username(gh)
            return gh
        except TwoFactorException:
            print(
                "Your account uses two-factor authentification. Please use a personal access token instead."
            )
            return login(args, method="Personal Access Token")
        except BadCredentialsException:
            print("Bad credentials")
            return login(args, method)


def github_username(gh: Github) -> str:
    return gh.get_user().login


def resolve_self_repository_shorthand(gh: Github, repo: str) -> str:
    """
    Returns adds USERNAME/ to a `repo` that has no slashes
    """
    if "/" not in repo:
        return github_username(gh) + "/" + repo
    return repo


def resolve_default_arguments(
    args: Dict[str, Any], repo_name: str, username: str
) -> Dict[str, Any]:
    """
    Resolves defaults for COLUMN and PROJECT using --default-* arguments
    ``repo_name`` must be of the form ``OWNER/REPO``
    
    >>> resolve_default_arguments(
    ...     { 'REPO': 'test', 'COLUMN': None, 'PROJECT': 'testy',
    ...     '--default-project': '1', '--default-column': '%(project)s' },
    ...     repo_name='ewen-lbh/project',
    ...     username='ewen-lbh',
    ... )
    { 'OWNER': 'ewen-lbh', 'REPO': 'test', 'COLUMN': 'testy', 'PROJECT': 'testy', '--default-project': '1', '--default-column': '%(project)s' }
    """
    placeholders = dict()
    placeholders["owner"], placeholders["repository"] = repo_name.split("/")
    placeholders["username"] = username
    args["PROJECT"] = args["PROJECT"] or (args["--default-project"] % (placeholders))
    placeholders["project"] = args["PROJECT"]
    args["COLUMN"] = args["COLUMN"] or (args["--default-column"] % (placeholders))

    return args


def push_to_repo(args: Dict[str, Any]) -> None:
    gh = login(args)
    repo_name = resolve_self_repository_shorthand(gh, args["REPO"])
    repo = gh.get_repo(repo_name)
    username = github_username(gh)
    args = resolve_default_arguments(args, repo_name, username)
    idea = args["IDEA"]
    project_name = args["PROJECT"]
    column_name = args["COLUMN"]
    assignees = args["--assign-to"] or (
        [username] if not args["--no-self-assign"] else []
    )

    # Get all labels
    all_labels = repo.get_labels()
    labels: List[Label] = []
    for label_name in args["--tag"]:
        not_found = label_name.lower() not in [t.name.lower() for t in all_labels]
        if not_found and args["--create-missing"]:
            if ask(
                q.Confirm(
                    "ans", message=f"Label {label_name!r} does not exist. Create it?"
                )
            ):
                label_data = q.prompt(
                    [
                        # TODO: Proper color prompt with color names, live color preview, default value that is removed once you start typing.
                        #       will need to use prompt-toolkit at some point.
                        # q.Text(
                        #     "color",
                        #     message="Choose a color for your label",
                        #     validate=validate_label_color,
                        #     default=lambda ans: f"{randint(0x0, 0xFFFFFF):6x}".upper()
                        # ),
                        q.Text(
                            "description",
                            message="A short description of your label",
                            default="",
                        ),
                    ]
                )

                color = randint(0x0, 0xFFFFFF)
                print(
                    "Creating label "
                    + dye(label_name, fg=color, style="reverse")
                    + " ..."
                )
                labels += [
                    repo.create_label(
                        name=label_name, color=f"{color:6x}", **label_data
                    )
                ]
            else:
                return
        elif not_found:
            print(error_message_no_object_found("label", label_name))
            return
        else:
            labels += [repo.get_label(label_name)]

    project = None
    for p in repo.get_projects():
        if p.name.lower() == project_name.lower():
            project = p
            break

    # Project not found
    if project is None and args["--create-missing"]:
        if ask(q.Confirm("ans", message=f"Create missing project {project_name!r}?")):
            description: str = ask(
                q.Text("ans", message="Enter the project's description...")
            )
            project = repo.create_project(name=project_name, body=description)
        else:
            return
    # Not found and not create
    elif project is None:
        print(error_message_no_object_found("project", project_name))
        return

    # Column
    column = None
    for c in project.get_columns():
        if c.name.lower() == column_name.lower():
            column = c
            break

    # Column not found
    if column is None and args["--create-missing"]:
        if ask(q.Confirm("ans", message=f"Create missing column {column_name!r}?")):
            column = project.create_column(column_name)
        else:
            return
    # Not found and not create
    elif column is None:
        print(error_message_no_object_found("column", column_name))
        return

    # Milestone
    milestone = None
    if args["--milestone"]:
        for m in repo.get_milestones():
            if m.title.lower() == args["--milestone"].lower():
                milestone = m
                break
        if milestone is None and args["--create-missing"]:
            if ask(
                q.Confirm(
                    "ans", message=f"Create missing milestone {args['--milestone']!r}?"
                )
            ):
                # TODO: Ask for a due date
                milestone = repo.create_milestone(title=args["--milestone"])
            else:
                return
        elif milestone is None:
            print(error_message_no_object_found("milestone", args["--milestone"]))
            return

    owner, repository = repo_name.split("/")

    if args["--issue"]:
        # Cant just use milestone=milestone because
        # create_issue(milestone=None) does not work, linter says.
        issue_creation_args = dict(
            title=args["--title"] or idea,
            body=idea if args["--title"] else "",
            assignees=assignees,
            labels=args["--tag"],
        )

        issue = None
        if not args["--dry-run"]:
            if milestone is not None:
                issue = repo.create_issue(**issue_creation_args, milestone=milestone)
            else:
                issue = repo.create_issue(**issue_creation_args)

            card = column.create_card(content_id=issue.id, content_type="Issue")
            url = issue.html_url
        else:
            if milestone is not None:
                print_dry_run(
                    f"repo.create_issue(**{issue_creation_args!r}, milestone={milestone!r})"
                )
            else:
                print_dry_run(f"repo.create_issue(**{issue_creation_args!r})")

            print_dry_run(
                f'column.create_card(content_id=issue.id, content_type="Issue")'
            )
            url = "N/A"

        print(
            make_github_issue_art(
                owner=owner,
                repository=repository,
                project=project.name,
                column=column.name,
                username=username,
                url=url,
                issue_number=(issue.number if issue is not None else "N/A"),
                labels=labels,
                body=args["IDEA"],
                title=args["--title"],
                assignees=assignees,
                milestone=(milestone.title if milestone is not None else None),
            )
        )

        if args["--open"] and not url == "N/A":
            webbrowser.open(url)
    else:

        card = None
        if not args["--dry-run"]:
            card = column.create_card(note=idea)
            url = project.html_url
        else:
            print_dry_run(f"column.create_card(note={idea!r}")
            url = "N/A"

        print(
            make_github_project_art(
                owner=owner,
                repository=repository,
                project=project_name,
                column=column_name,
                body=args["IDEA"],
                url=url,
            )
        )

        # Open project URL
        if args["--open"] and url != "N/A":
            webbrowser.open(url)


def push_to_user(args: Dict[str, Any]) -> None:
    gh = login(args)
    idea = args["IDEA"]
    project_name: str = args["--user-project"]
    column_name: str = args["PROJECT"]
    username = github_username(gh)
    print(
        f"Saving card in {dye(username, C_PRIMARY)} › {dye(project_name, C_PRIMARY)} › {dye(column_name, C_PRIMARY)}..."
    )
    project = None
    user = gh.get_user(github_username(gh))
    for p in user.get_projects():
        if p.name.lower() == project_name.lower():
            project = p
            break

    # Project not found
    if project_name is None and args["--create-missing"]:
        if ask(q.Confirm("ans", message=f"Create missing project {project_name!r}?")):
            description = ask(
                q.Editor("ans", message="Enter the project's description...")
            )
            project = user.create_project(name=project_name, body=description)
        else:
            return
    # Not found and not create
    elif project_name is None:
        print(error_message_no_object_found("project", project_name))
        return

    # Column
    column = None
    for c in project.get_columns():
        if c.name.lower() == column_name.lower():
            column = c
            break

    # Column not found
    if column_name is None and args["--create-missing"]:
        if ask(q.Confirm("ans", message=f"Create missing column {column_name!r}?")):
            column = project.create_column(column_name)
        else:
            return
    # Not found and not create
    elif column_name is None:
        print(error_message_no_object_found("column", column_name))
        return

    if not args["--dry-run"]:
        column.create_card(note=idea)
        url = project.html_url
    else:
        print_dry_run(f"column.create_card(note={idea!r})")
        url = "N/A"

    print(
        make_github_user_project_art(
            username=username,
            project=project_name,
            column=column_name,
            body=args["IDEA"],
            url=url,
        )
    )

    # Open project URL
    if args["--open"] and url != "N/A":
        webbrowser.open(url)


if __name__ == "__main__":
    import doctest

    doctest.testmod()



from typing import *
from ideaseed.constants import C_PRIMARY, RELEASES_RSS_URL
from xml.dom.minidom import parseString as parse_xml
import cli_box
import requests
from ideaseed.utils import ask, dye
import inquirer as q
from semantic_version import Version
import subprocess
import re


def get_latest_version() -> Version:
    raw_rss = requests.get(RELEASES_RSS_URL).text
    rss = parse_xml(raw_rss)
    version = (
        rss.childNodes[0]
        .getElementsByTagName("channel")[0]
        .getElementsByTagName("item")[0]
        .getElementsByTagName("title")[0]
        .childNodes[0]
        .nodeValue
    )
    return Version(version)


def get_changelog_heading_anchor(release_notes: str, upgrade_to: Version) -> str:
    """
    Get the changelog heading anchor for github.
    >>> get_changelog_heading_anchor(Version('0.8.0'))
    '080---2020-06-20'
    """
    pattern = re.compile(r"## \[" + str(upgrade_to) + r"\] - (.+)")
    date = pattern.search(release_notes).group(1)
    return f"{str(upgrade_to).replace('.', '')}---{date}"


def get_release_notes() -> str:
    return requests.get(
        "https://raw.githubusercontent.com/ewen-lbh/ideaseed/master/CHANGELOG.md"
    ).text


def get_release_notes_for_version(release_notes: str, version: Version) -> str:
    in_target_version = False
    ret = ""
    for line in release_notes.split("\n"):
        if line.startswith(f"## [{version}]"):
            in_target_version = True
            continue  # Don't add the actual heading to the release notes for this version
        # If the line is the start of another version's section, set to false
        elif line.startswith(f"##") and not line.startswith("###"):
            in_target_version = False

        if in_target_version:
            ret += line + "\n"
    return ret
    
def get_versions_list_from_release_notes(release_notes: str) -> List[Version]:
    # Declarations
    pattern = re.compile(r'\[Unreleased\]\: https\:\/\/github\.com\/ewen\-lbh\/ideaseed\/compare\/v\d+\.\d+\.\d+\.\.\.HEAD')
    extract_version_pattern = re.compile(r'\[(\d+\.\d+\.\d+)\]: https://')
    version_strings: List[str] = []
    # Tracking varialbe
    in_links_section = False
    for line in release_notes.splitlines():
        # Line is [Unreleased]: https://..., which always begins the version list
        if pattern.match(line):
            in_links_section = True
            continue
        # Subsequent lines that are like [version]: https://... are those we get versions from
        if in_links_section and extract_version_pattern.match(line):
            version_strings += [extract_version_pattern.search(line).group(1)]
    # Parse into SemVer Version objects
    versions = [ Version(v) for v in version_strings ]
    # Sort them (from 0.0.0 to ∞.∞.∞)
    versions = sorted(versions, key=lambda v: v.precedence_key)
    return versions

def get_release_notes_between_versions(release_notes: str, version_from: Version, version_to: Version) -> str:
    # Get every version ∈ (version_from, version_to]
    versions = [v for v in get_versions_list_from_release_notes(release_notes) if version_from < v <= version_to]
    catd_release_notes = ""
    for version in versions:
        catd_release_notes += f"## {version}"
        catd_release_notes += get_release_notes_for_version(release_notes, version)
    return catd_release_notes

def get_release_notes_link(release_notes: str, upgrade_to: Version) -> str:
    anchor = get_changelog_heading_anchor(release_notes, upgrade_to)
    return f"https://github.com/ewen-lbh/ideaseed/tree/master/CHANGELOG.md#{anchor}"


def notification(upgrade_from: Version, upgrade_to: Version) -> str:
    return cli_box.rounded(
        f"""==== Update available! ====

A new version of ideaseed is available for download:
{upgrade_from} -> {upgrade_to}
"""
    )


def render_markdown(text: str) -> str:
    heading = re.compile(r"(#+)\s*(.+)")
    list_item = re.compile(r"(\s*)-\s*(.+)")
    image = re.compile(r"!\[(.+)\]\((.+)\)")
    code = re.compile(r"`([^`]+)`")
    # link = re.compile(r'\[(.+)\]\((.+)\)')
    rendered = ""
    for line in text.splitlines():
        if heading.match(line):
            match = heading.search(line)
            rendered_line = dye(match.group(2), style="bold")
        elif list_item.match(line):
            match = list_item.search(line)
            rendered_line = match.group(1) + dye("• ", style="dim") + match.group(2)
        else:
            rendered_line = line
        rendered_line = image.sub(dye(r"(image: \1)", style="dim"), rendered_line)
        rendered_line = code.sub(dye(r" \1 ", bg=0xDEDEDE), rendered_line)
        # rendered_line = link.sub(dye(r' (link: \1)', style="dim"), rendered_line)
        rendered += rendered_line + "\n"
    return rendered


def prompt(upgrade_from: Version, upgrade_to: Version) -> bool:
    """
    Returns ``True`` if the user wants to upgrade, ``False`` otherwise.
    """
    answer = ask(
        q.List(
            "ans",
            message=f"Upgrade to v{upgrade_to} now?",
            choices=["Yes", "What has changed?", "No"],
        )
    )
    if answer == "What has changed?":
        release_notes = get_release_notes()
        url = get_release_notes_link(release_notes, upgrade_to)
        all_versions = get_versions_list_from_release_notes(release_notes)
        # If the version jump is more than one version, print concatednated release notes
        # so that the user can get all of the changes.
        # eg: i'm upgrading from 0.6.0 to 0.10.0, but there has been 0.8.0 and 0.9.0 in between,
        #     i want all the changes, not just the ones from 0.9.0 to 0.10.0
        if len([v for v in all_versions if upgrade_from < v <= upgrade_to]) > 1:
            notes = get_release_notes_between_versions(release_notes, upgrade_from, upgrade_to)
        # else just get the single one.
        # this is because doing get_release_notes_between_versions would still return 
        # the version <h2>, which would be stupid to show here
        else:
            notes = get_release_notes_for_version(release_notes, upgrade_to)
        print(
            f"""\
Release notes for v{upgrade_from} -> v{upgrade_to}
====================================

{render_markdown(notes).strip()}

---------------------------------------------
To see images, you can also read this online:
{url}
"""
        )
        return ask(q.Confirm("ans", message="Upgrade now?"))
    else:
        return answer == "Yes"


def upgrade(upgrade_to: Version):
    cmd = ["pip", "install", "--upgrade", f"ideaseed=={upgrade_to}"]
    print(f"Running {' '.join(cmd)}...")
    subprocess.run(cmd)



"""Note down your ideas and get them to the right place, without switching away from your terminal

Usage: 
    ideaseed (--help | --about | --version)
    ideaseed [options] [-t TAG...] [-l LABEL...] [-# LABEL...] [-@ USERNAME...] ARGUMENTS...

Examples:
    # Save a card "test" in schoolsyst/webapp > project "UX" > column "To-Do"
    $ ideaseed schoolsyst/webapp UX "test"
    # Save a card "lorem" in your-username/ipsum > project "ipsum" > column "To-Do"
    $ ideaseed ipsum "lorem"
    # Save a card "a CLI to note down ideas named ideaseed" in your user profile > project "incubator" > column "willmake"
    $ ideaseed --user-keyword=project --user-project=incubator project willmake "a CLI to note down ideas named ideaseed"

Arguments:
    REPO        Select a repository
                    If not given                        Uses Google Keep instead of GitHub (or uses your user profile's projects if --project is used)
                    If --user-keyword's value is given  Creates a card on your user's project (select which project with --user-project)
                    If given in the form OWNER/REPO     Uses the repository OWNER/REPO
                    If given in the form REPO           Uses the repository "your username/REPO"
    PROJECT     Select a project by name to put your card to [default: --default-project's value]
                    If creating a card on your user's project, this becomes the COLUMN
    COLUMN      Select a project's column by name [default: --default-column's value]
                    If creating a card on your user's project, this is ignored

Options:
    -c --color COLOR           Chooses which color to use for Google Keep cards. See Color names for allowed values.
    -t --tag TAG               Adds tags to the Google Keep card or labels to the GitHub issue (see --issue). Use it multiple times to set multiple tags.
    -i --issue                 Creates an issue for the card and link them together. IDEA becomes the issue's title, except if --title is specified,
                               in which case IDEA becomes the issue's description and --title's value the issue title.
    (WIP) -I --interactive     Prompts you for the above options when they are not provided.
    -T --title TEXT            Sets the Google Keep card's title. When used with --issue, sets the issue's title.
    -L --logout                Clears the authentification cache
    -m --create-missing        Create non-existant tags, labels, projects or columns specified, upon confirmation.
    -o --open                  Open the relevant URL in your web browser.
    -l --label LABEL           Alias for --tag. See --tag's description.
    -# --label LABEL           Just another shorthand for --label.
    -@ --assign-to USERNAME    Assigns users to the created issue. Only works when --issue is used.
    -M --milestone TITLE       Assign the issue to a milestone with title TITLE.
       --pin                   Pin Google Keep cards
       --dry-run               Don't actually affect websites, put print what it would've done. Still logs in. Beware, --create-missing ignores this flag and still creates missing objects.
       --about                 Details about ideaseed like currently-installed version
       --version               Like --about, without dumb and useless stuff

Settings options: It's comfier to set these in your alias, e.g. alias idea="ideaseed --user-project=incubator --user-keyword=project --no-auth-cache --create-missing"
       --user-project NAME     Name of the project to use as your user project
       --user-keyword NAME     When REPO is NAME, creates a GitHub card on your user profile instead of putting it on REPO
       --no-auth-cache         Don't save credentials in a temporary file at {cache_filepath}
       --no-check-for-updates  Don't check for updates, don't prompt to update when current version is outdated
       --no-self-assign        Don't assign the issue to yourself
    
    Options beginning with --default can refer to some dynamic information using %(placeholder)s.
    Available placeholders:
        repository - the repository's name
        owner      - the repository's owner
        username   - the username of which account you are using ideaseed with
        project    - the selected project's name (only available to --default-column)
    
       --default-column NAME   Set the default column name. [default: To Do]
       --default-project NAME  Set the default project name. [default: %(repository)s]
       

Color names: Try with the first letter only too
    blue, brown, darkblue, gray, green, orange, pink, purple, red, teal, white, yellow
    Some color have aliases:
    - cyan is the same as teal
    - indigo is the same as darkblue
    - grey is the same as gray
    - magenta is the same as purple
"""

from ideaseed.update_checker import get_latest_version
from ideaseed import update_checker
from ideaseed.gkeep import push_to_gkeep
from ideaseed.github import clear_auth_cache, push_to_repo, push_to_user
from typing import *
from colr import docopt
from pprint import pprint
from ideaseed.utils import ask, dye, get_token_cache_filepath
from ideaseed.constants import (
    COLOR_ALIASES,
    COLOR_NAME_TO_HEX_MAP,
    VALID_COLOR_NAMES,
    VERSION,
)
from ideaseed.dumb_utf8_art import ABOUT_SCREEN
import cli_box as box
from inquirer import Confirm
import subprocess
import sys


def run(argv=None):
    args = resolve_arguments(
        docopt(
            __doc__.format(cache_filepath=get_token_cache_filepath("*")),
            argv,
            script="ideaseed",
        )
    )
    validate_argument_presence(args)

    args["--tag"] += args["--label"]
    # Remove duplicate tags
    args["--tag"] = list(
        set(args["--tag"])
    )  # XXX: We're loosing order of elements here.

    if not args["--no-check-for-updates"]:
        latest_version = get_latest_version()
        if latest_version > VERSION:
            print(update_checker.notification(VERSION, latest_version))
            if update_checker.prompt(VERSION, latest_version):
                update_checker.upgrade(latest_version)
                print(f"Re-running your command with ideaseed v{latest_version}...")
                print("Running " + " ".join(sys.argv))
                subprocess.run(sys.argv)
                return

    if args["--dry-run"]:
        print(
            dye(
                f"\n============ Running in {dye('dry run', style='bold', fg=0xfff)} {dye('mode ============', style='dim')}\n",
                style="dim",
            )
        )

    if args["--about"]:
        print(ABOUT_SCREEN.format(version=VERSION))
        return

    if args["--version"]:
        print(VERSION)
        return

    if args["--color"]:
        args["--color"] = expand_color_name(args["--color"])

    if args["--logout"]:
        clear_auth_cache()

    if args["REPO"]:
        if args["REPO"] == args["--user-keyword"]:
            push_to_user(args)
        else:
            push_to_repo(args)
    else:
        push_to_gkeep(args)


def resolve_arguments(args: Dict[str, Any]) -> Dict[str, Any]:
    """
    Apparently required positional arguments must come first...
    So I'm resolving them by hand here.
    """
    positional_args = args["ARGUMENTS"]
    idea = repo = project = column = None
    if len(positional_args) == 1:
        idea = positional_args[0]
    if len(positional_args) == 2:
        repo, idea = positional_args
    if len(positional_args) == 3:
        repo, project, idea = positional_args
    if len(positional_args) >= 4:
        repo, project, column, idea = positional_args
    return {"REPO": repo, "PROJECT": project, "COLUMN": column, "IDEA": idea, **args}


class ValidationError(Exception):
    pass


def validate_argument_presence(args: Dict[str, str]) -> None:
    """
    Raises a `ValidationError` if one of the arguments is not allowed based
    on the other arguments.
    """

    GOOGLE_KEEP_ONLY = ("--color",)
    GITHUB_ONLY = ("--issue",)
    GITHUB_ISSUE_ONLY = ("--milestone", "--assign-to")
    using_github = len(args["ARGUMENTS"]) > 1

    if using_github and not args["--issue"] and (args["--tag"] or args["--label"]):
        raise ValidationError(
            "--label (or --tag) can only be used with --issue or when creating"
            "a Google Keep card."
        )

    if not args["--issue"] and any(
        [v for k, v in args.items() if k in GITHUB_ISSUE_ONLY]
    ):
        raise ValidationError(
            "The following options only work when --issue is used: "
            + ", ".join(GITHUB_ISSUE_ONLY)
        )

    if using_github and any([v for k, v in args.items() if k in GOOGLE_KEEP_ONLY]):
        raise ValidationError(
            "The following options are not allowed when using GitHub: "
            + ", ".join(GOOGLE_KEEP_ONLY)
        )
    if not using_github and any([v for k, v in args.items() if k in GITHUB_ONLY]):
        raise ValidationError(
            "The following options are not allowed when using Google Keep: "
            + ", ".join(GITHUB_ONLY)
        )


def expand_color_name(color: str) -> str:
    # Initialize the array of matches
    matching_color_names: List[str] = []
    # Filter `color_names` to only get the color names that start with `color`
    for color_name in VALID_COLOR_NAMES:
        if color_name.lower().startswith(color.lower()):
            matching_color_names += [color_name]
    # Resolve synonyms
    matching_color_names_resolved = matching_color_names
    for i, color_name in enumerate(matching_color_names):
        if color_name in COLOR_ALIASES.keys():
            matching_color_names_resolved[i] = COLOR_ALIASES[color_name]
    # Remove duplicates
    matching_color_names = list(set(matching_color_names))
    # If we have exactly _one_ element of `color_names` that matches, we return the only
    # element: it's a match!
    if len(matching_color_names) == 1:
        return matching_color_names[0]
    # If we have multiple choices, the given color was ambiguous.
    if len(matching_color_names) > 1:
        matching_color_names_display: List[str] = []
        for matching_color_name in matching_color_names:
            matching_color_names_display += [
                dye(
                    matching_color_name,
                    fg=0x000,
                    bg=COLOR_NAME_TO_HEX_MAP[matching_color_name],
                )
            ]
        raise ValueError(
            f"Ambiguous color shorthand {color!r}, it could mean one of: "
            + ", ".join(matching_color_names_display)
        )
    # If we had an empty array, then the given color name was incorrect.
    raise ValueError(f"Unknown color {color}")



from ideaseed.dumb_utf8_art import make_google_keep_art
from ideaseed.constants import COLOR_NAME_TO_HEX_MAP, C_PRIMARY
import json
import webbrowser
from os import path
from ideaseed.utils import (
    ask,
    dye,
    error_message_no_object_found,
    get_token_cache_filepath,
    print_dry_run,
)
from typing import *
from inquirer import Text, Password, Confirm
from gkeepapi import Keep
from gkeepapi.exception import LoginException, APIException
from gkeepapi.node import ColorValue
import sys
import cli_box


def write_to_cache(keep: Keep, email: str) -> None:
    with open(get_token_cache_filepath("gkeep"), "w", encoding="utf8") as file:
        json.dump({"master_token": keep.getMasterToken(), "email": email}, file)


def login_from_cache() -> Optional[Keep]:
    if not path.exists(get_token_cache_filepath("gkeep")):
        return None

    with open(get_token_cache_filepath("gkeep"), encoding="utf8") as file:
        creds = json.load(file)

    keep = Keep()
    keep.resume(email=creds["email"], master_token=creds["master_token"])
    return keep


def login(args: Dict[str, Any]) -> Keep:
    # Try to log in from cache
    keep = login_from_cache()
    if keep is not None:
        return keep
    else:
        del keep

    # Ask for creds
    username, password = ask(
        Text("u", message="E-mail"), Password("p", message="Password")
    )

    # Log in
    keep = Keep()
    try:
        keep.login(username, password)
        write_to_cache(keep, username)
    except LoginException as error:
        # Handle errors...
        (topic, message) = error.args
        if topic == "BadAuthentification":
            print(dye("Bad credentials", fg=0xF00))
            return login(args)
        elif topic == "NeedsBrowser":
            print(
                dye(
                    """You have two-step authentification set up, please add an App Password.
Go to https://myaccount.google.com/apppasswords,
Click on 'Generate', Choose a name and a device, then copy the code
and use it as your password.""",
                    fg=0xF00,
                )
            )
            sys.exit()
    return keep


def push_to_gkeep(args: Dict[str, Any]) -> None:
    # Log in
    print("🔑 Logging in...")
    sys.stdout.flush()
    # Handle API errors
    try:
        keep = login(args)
    except APIException as error:
        print("❌ Error with the Google Keep API")
        if error.code == 429:
            print(
                """Too much requests per minute. Try again later.
Don't worry, your idea is still safe,
just up-arrow on your terminal to re-run the command :)"""
            )
        print(dye(error, style="dim"))
        return
    print("✅ Logged in.")

    color = args["--color"] or "White"
    note = None

    # Find/create all the labels
    labels = []
    for tag in args["--tag"]:
        label = keep.findLabel(tag)
        if label is None and args["--create-missing"]:
            if ask(Confirm("ans", message=f"Create missing tag {tag!r}?")):
                label = keep.createLabel(tag)
        elif label is None:
            print(error_message_no_object_found("tag", tag))
            return
        labels += [label]
    
    # Create the card
    if not args["--dry-run"]:
        note = keep.createNote(title=args["--title"], text=args["IDEA"])
        note.color = getattr(ColorValue, color)
        note.pinned = args["--pin"]
        url = f"https://keep.google.com/u/0/#NOTE/{note.id}"
        for label in labels:
            note.labels.add(label)
    else:
        print_dry_run(
            f"note = keep.createNote(title={args['--title']!r}, text={args['IDEA']!r})"
        )
        print_dry_run(f"note.color = getattr(ColorValue, {color!r})")
        print_dry_run(f"note.pinned = {args['--pin']!r}")
        url = "N/A"
        for label in labels:
            print_dry_run(f"note.labels.add({label!r})")

    # Announce created card
    print(
        make_google_keep_art(
            url=url,
            title=args["--title"],
            pinned=args["--pin"],
            tags=args["--tag"],
            body=args["IDEA"],
            color=args["--color"],
        )
    )

    # Beam it up to Google's servers
    keep.sync()

    # Open the browser
    if args["--open"] and not args['--dry-run']:
        webbrowser.open(url)



from typing import Optional
from typing import *
import colr
import inquirer
from os import path
from random import randint


def dye(
    text: str,
    fg: Optional[int] = None,
    bg: Optional[int] = None,
    style: Optional[str] = None,
    no_closing: bool = False,
):
    return colr.color(
        text=text,
        fore=f"{fg:x}" if fg is not None else None,
        back=f"{bg:x}" if bg is not None else None,
        style=style,
        no_closing=no_closing,
    )


def readable_text_color_on(
    background: int, light: int = 0xFFFFFF, dark: int = 0x000000
) -> int:
    """
    Choses either ``light`` or ``dark`` based on the background color
    the text is supposed to be written on ``background`` (also given as an hex int)
    
    WARN: All the color ints must be exactly 6 digits long.
    
    >>> readable_text_color_on(0xFEFAFE)
    0
    >>> readable_text_color_on(0x333333)
    16777215
    """
    r, g, b = hex_to_rgb(f"{background:6x}")
    luminance = (0.299 * r + 0.587 * g + 0.114 * b) / 255
    if luminance > 0.5:
        return dark
    else:
        return light


def hex_to_rgb(hexstring: str) -> Tuple[int, int, int]:
    """
    Converts a hexstring (without initial '#') ``hexstring`` into a 
    3-tuple of ints in [0, 255] representing an RGB color
    
    >>> hex_to_rgb('FF00AA')
    (255, 0, 170)
    """
    return tuple(int(hexstring[i : i + 2], 16) for i in (0, 2, 4))  # skip '#'


def get_random_color_hexstring() -> str:
    return f"{randint(0x0, 0xFFFFFF):6x}".upper()


def ask(*questions) -> Union[List[Any], Any]:
    if len(questions) == 1:
        # No need to turn the hash into a tuple, just return the only value
        questions[0].name = "ans"
        return inquirer.prompt(questions)["ans"]

    for i, _ in enumerate(questions):
        questions[i].name = i
    # Ask dem questions
    answers = inquirer.prompt(questions)
    # Turn into a list of tuple
    answers = list(answers.items())
    # Sort by key
    answers = sorted(answers, key=lambda a: a[0])
    # Get only the answers
    answers = [a[1] for a in answers]
    return answers


def get_token_cache_filepath(service: str) -> str:
    return path.join(path.dirname(path.dirname(__file__)), f".auth-cache--{service}")


def english_join(items: List[str]) -> str:
    """
    Joins items in a sentence-compatible way, adding "and" at the end
    
    >>> english_join(["a", "b", "c"])
    'a, b and c'
    >>> english_join(["a"])
    'a'
    >>> english_join(["a", "b"])
    'a and b'
    """
    if len(items) == 1:
        return items[0]
    return ", ".join(items[: len(items) - 1]) + " and " + items[len(items) - 1]


def print_dry_run(text: str):
    """
    Apply special formatting for dry-run specific messages
    """
    DRY_RUN_FMT = dye(" DRY RUN ", bg=0x333, fg=0xFFF) + dye("  {}", style="dim")
    print(DRY_RUN_FMT.format(text))


def error_message_no_object_found(objtype: str, objname: str) -> str:
    return (
        dye(f"Error: missing {objtype} {objname!r}", fg=0xF00,)
        + """
💡 Use --create-missing and ideaseed will ask you if you want to create missing 
labels, issues, projects, columns, milestones..."""
    )


if __name__ == "__main__":
    import doctest

    doctest.testmod()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ideaseed']

package_data = \
{'': ['*']}

install_requires = \
['cli-box>=0.2.0,<0.3.0',
 'colr>=0.9.1,<0.10.0',
 'docopt>=0.6.2,<0.7.0',
 'flatten-list>=0.2.0,<0.3.0',
 'gkeepapi>=0.11.16,<0.12.0',
 'inquirer>=2.7.0,<3.0.0',
 'pygithub>=1.51,<2.0',
 'semantic-version>=2.8.5,<3.0.0',
 'strip-ansi>=0.1.1,<0.2.0']

entry_points = \
{'console_scripts': ['ideaseed = ideaseed.cli:run']}

setup_kwargs = {
    'name': 'ideaseed',
    'version': '0.10.0',
    'description': 'Note down your ideas and get them to the right place, without switching away from your terminal',
    'long_description': '<center><p align="center">\n<img alt="ideaseed" src="visual-identity/ideaseed-logomark-color-transparent.svg" width="550px">\n</p></center>\n\nDo you have ideas suddenly and just wished you could catch them as fast as possible, as to not loose them, without having to interrupt what you were doing?\n\nAs I guy with a lot of more or less stupid ideas, I use Google Keep as a centralized place to put all of my thoughts that I deem worthy of consideration.\n\nI recently started to use GitHub Projects for _coding_ project ideas as a [single project called "incubator" on my GitHub profile directly](https://github.com/ewen-lbh?tab=projects), and as issues or notes when the idea is related to an already-existing project and repo.\n\nBut when I don\'t get to decide _when_ this idea comes, and I often need to interrupt what am I doing to open github, get to the right page, input my idea and get back. And I find it frustrating.\n\nEnough rambling. Here\'s what you came for.\n\nNote down your ideas and get them to the right place, without switching away from your terminal\n\n## Installation\n\nIdeaseed is available [on PyPI.org](https://pypi.org/project/ideaseed):\n\n```sh-session\npip install ideaseed\n```\n\nSee [Installation troubleshooting](#installation-troubleshooting) if you can\'t install it like this.\n\n## Updating\n\nExcept if you specify `--no-check-updates`, the program will automatically check for new versions and ask to upgrade for you (since [v0.4.0]). If you want to upgrade manually, simply run\n\n```sh-session\npip install -U ideaseed\n```\n\nTo see the version history, you can read the [changelog](https://github.com/ewen-lbh/ideaseed/tree/master/CHANGELOG.md), though, since [v0.9.0], you can read the release notes straight from your terminal by choosing "What has changed?" when the "Upgrade available" notification asks you if you want to update.\n\n## Usage\n\n```bash\nideaseed (--help | --about | --version)\nideaseed [options] ARGUMENTS...\n```\n\n### Examples\n\n```sh-session\n# Save a card "test" in schoolsyst/webapp > project "UX" > column "To-Do"\n$ ideaseed schoolsyst/webapp UX "test"\n# Save a card "lorem" in your-username/ipsum > project "ipsum" > column "To-Do"\n$ ideaseed ipsum "lorem"\n# Save a card "a CLI to note down ideas named ideaseed" in your user profile > project "incubator" > column "willmake"\n$ ideaseed --user-keyword=project --user-project=incubator project "a CLI to note down ideas named ideaseed"\n```\n\n### Arguments\n\n| Argument  | Meaning                                                                                                              | Default value  |\n| --------- | -------------------------------------------------------------------------------------------------------------------- | -------------- |\n| `REPO`    | Select a repository by name                                                                                          |\n|           | If not given, uses Google Keep instead of GitHub (or uses your user profile\'s projects if --project is used)         |\n|           | If --user-keyword\'s value is given, creates a card on your user\'s project (select which project with --user-project) |\n|           | If given in the form OWNER/REPO, uses the repository OWNER/REPO                                                      |\n|           | If given in the form REPO, uses the repository "your username/REPO"                                                  |\n| `PROJECT` | Select a project by name to put your card to [default: REPO\'s value]                                                 | `REPO`\'s value |\n|           | If creating a card on your user\'s project, this becomes the COLUMN                                                   |\n| `COLUMN`  | Select a project\'s column by name [default: To-Do]                                                                   | To-Do          |\n|           | If creating a card on your user\'s project, this is ignored                                                           |\n\n### Options\n\n| Shorthand | Full-length              | Description                                                                                                     |\n| --------- | ------------------------ | --------------------------------------------------------------------------------------------------------------- |\n| `-c`      | `--color COLOR`          | Chooses which color to use for Google Keep cards. See [Color names](#color-names) for a list of valid values    |\n| `-t`      | `--tag TAG`              | Adds tags to the Google Keep card. Can also be used on GitHub together with --issue to add labels to the issue. |\n| `-i`      | `--issue TITLE`          | Creates an issue with title TITLE.                                                                              |\n| `-L`      | `--logout`               | Clears the authentification cache                                                                               |\n| `-m`      | `--create-missing`       | Create non-existant tags, projects or columns specified (needs confirmation if -I is used)                      |\n| `-o`      | `--open`                 | Open the relevant URL in your web browser.                                                                      |\n| `-l`      | `--label LABEL`          | Alias for --tag. See --tag\'s description.                                                                       |\n| `-@`      | `--assign-to USERNAME`   | Assigns users to the created issue. Only works when --issue is used.                                            |\n| `-M`      | `--milestone TITLE`      | Assign the issue to a milestone with title TITLE.                                                               |\n|           | `--pin`                  | Pin the Google Keep card                                                                                        |\n|           | `--about`                | Details about ideaseed like currently-installed version                                                         |\n|           | `--version`              | Like --about, without dumb and useless stuff                                                                    |\n|           | `--user-project NAME`    | Name of the project to use as your user project                                                                 |\n|           | `--user-keyword NAME`    | When REPO is NAME, creates a GitHub card on your user profile instead of putting it on REPO                     |\n|           | `--no-auth-cache`        | Don\'t save credentials in a temporary file                                                                      |\n|           | `--no-check-for-updates` | Don\'t check for updates, don\'t prompt to update when current version is outdated                                |\n|           | `--no-self-assign`       | Don\'t assign the issue to yourself                                                                              |\n\n#### Color names\n\n- blue\n- brown\n- darkblue\n- gray\n- green\n- orange\n- pink\n- purple\n- red\n- teal\n- white\n- yellow\n\nYou don\'t have to specify the whole color name, just enough to be non-ambiguous:\n\n- bl\n- br\n- d\n- gra\n- gre\n- o\n- pi\n- pu\n- r\n- t\n- w\n- y\n\nSome color have aliases:\n\n- cyan is the same as teal\n- indigo is the same as darkblue\n- grey is the same as gray\n- magenta is the same as purple\n\n#### Relax. You don\'t need to remember those options (Coming in [v1.0.0])\n\nYou can also use `ideaseed -I` to prompt you for some information:\n\n- Where do you want to upload this idea? (github, google keep)\n- If you decide to use github,\n  - On your profile?\n  - If not:\n    - Which repo? (using REPO or OWNER/REPO) (autocompletes with repositories you contribute to)\n    - Which column? (choices are the column names, and you can type the column\'s index to be quicker)\n- If you decide to use google keep,\n  - Which color? (defaults to white)\n  - Some tags?\n\n## Installation troubleshooting\n\nIf you get an error message saying "No matching distribution found":\n\n```sh-session\n$ pip install ideaseed\nCollecting ideaseed\n  Could not find a version that satisfies the requirement ideaseed (from versions: )\nNo matching distribution found for ideaseed\n```\n\nSee if the python version `pip` uses is at least 3.6:\n\n```sh-session\n$ pip --version\npip 9.0.1 from /usr/lib/python2.7/dist-packages (python 2.7) # Should be at least "(python 3.6)"\n```\n\nYou can then try with `pip3` (`pip3 --version` should report a python version of at least 3.6):\n\n```sh-session\n$ pip3 --version\npip 20.0.2 from /home/ewen/.local/lib/python3.7/site-packages/pip (python 3.7)\n$ pip3 install ideaseed\n```\n\n[v1.0.0]: https://github.com/ewen-lbh/ideaseed/milestone/2\n[v0.4.0]: https://github.com/ewen-lbh/ideaseed/tree/master/CHANGELOG.md#040---2020-06-18\n[v0.9.0]: https://github.com/ewen-lbh/ideaseed/blob/master/CHANGELOG.md#090---2020-06-22\n\n* * *\n\n<center><p align="center">\n  <img src="visual-identity/ideaseed-logo-black-transparent.png" width="40px">\n  <br>\n  ideaseed by <a href="https://ewen.works">Ewen Le Bihan</a>\n</p></center>\n',
    'author': 'Ewen Le Bihan',
    'author_email': 'ewen.lebihan7@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ewen-lbh/ideaseed',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
